@@ -1,4 +1,4 @@
-/* $Id: acl.h,v 1.29 2007/10/05 10:08:08 manu Exp $ */
+/* $Id: acl.h,v 1.29.2.1 2008/08/22 21:44:16 manu Exp $ */
 
 /*
  * Copyright (c) 2004-2007 Emmanuel Dreyfus
@@ -149,6 +149,7 @@ struct acl_param {
 #define A_FREE_ECODE		0x04
 #define A_FREE_MSG		0x08
 #define A_FREE_REPORT		0x10
+#define A_DROP_ACL		0x40
 
 struct all_list_entry;
 
@@ -223,6 +224,7 @@ char *stage_string(acl_stage_t);
 struct acl_clause_rec *get_acl_clause_rec(acl_clause_t);
 struct acl_clause_rec *acl_list_item_fixup(acl_clause_t, acl_clause_t);
 void acl_init(void);
+void acl_drop(void);
 void acl_clear(void);
 void acl_add_clause(acl_clause_t, void *);
 void acl_negate_clause(void);
