@@ -1,7 +1,7 @@
-/* $Id: milter-greylist.h,v 1.37 2005/01/29 18:42:53 manu Exp $ */
+/* $Id: milter-greylist.h,v 1.69 2007/10/03 10:52:23 manu Exp $ */
 
 /*
- * Copyright (c) 2004 Emmanuel Dreyfus
+ * Copyright (c) 2004-2007 Emmanuel Dreyfus
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,19 +33,30 @@
 #define _MILTER_GREYLIST_H_
 
 #include <sys/socket.h>
-#include <arpa/inet.h>
 #include <netinet/in.h>
+#include <arpa/inet.h>
 #include <netdb.h>
 
 #include <libmilter/mfapi.h>
 #include "config.h"
 #include "dump.h"
 
+/* environment of Solaris workaround for stdio descriptor limitation */
+#include "fd_pool.h"
+
 #define NUMLEN 20
-#define PATHLEN 1024
+#define QSTRLEN 1024
 #define REGEXLEN 1024
 #define HDRLEN 1024
 #define HEADERNAME "X-Greylist"
+/* 
+ * Maximum URL length. This is just a hint, 
+ * the code will adjust the buffer if needed.
+ */
+#define URLMAXLEN	2083
+
+LIST_HEAD(urlchecklist, urlcheck_entry);
+
 
 
 #if defined(HAVE_GETNAMEINFO)
@@ -79,24 +90,125 @@ typedef union {
 #define SADDR6(sa)	(&SA6(sa)->sin6_addr)
 #endif
 
+/* Notes:
+ * -For IPv6 not using s6_addr32 as Solaris 8 for some reason has it only 
+ *  defined for its kernel... 
+ * -Using also first two characters in "from" and "rcpt" to distribute 
+ *  potentially lot of triplets coming from a single host (first two chars 
+ *  only because "<>" is the "shortest" email address)
+ */
+#define F2B(s) (tolower((int)*(s)) | (tolower((int)*((s)+1)) << 8))
+#define F2B_SPICE(from, rcpt) (conf.c_lazyaw ? 0 : (F2B(from) ^ F2B(rcpt)))
+
+#define BUCKET_HASH_V4(v4a, v4m, from, rcpt, bucket_count) 	\
+  ((ntohl((v4a)->s_addr & (v4m)->s_addr)			\
+    ^ F2B_SPICE(from, rcpt))					\
+   % bucket_count) 
+
+#ifdef AF_INET6
+#define IN6CAST32(_a) ((uint32_t *)(&(_a)->s6_addr))
+
+#define BUCKET_HASH_V6(v6a, v6m, from, rcpt, bucket_count)	\
+  ((ntohl(IN6CAST32(v6a)[0] & IN6CAST32(v6m)[0]) ^		\
+    ntohl(IN6CAST32(v6a)[1] & IN6CAST32(v6m)[1]) ^		\
+    ntohl(IN6CAST32(v6a)[2] & IN6CAST32(v6m)[2]) ^		\
+    ntohl(IN6CAST32(v6a)[3] & IN6CAST32(v6m)[3])		\
+    ^ F2B_SPICE(from, rcpt))					\
+   % bucket_count)
+
+#define BUCKET_HASH(sa, from, rcpt, bucket_count)		\
+  (sa->sa_family == AF_INET ?					\
+   BUCKET_HASH_V4(SADDR4(sa), 					\
+		  &conf.c_match_mask,				\
+		  from, rcpt, bucket_count)			\
+   : sa->sa_family == AF_INET6 ? 				\
+   BUCKET_HASH_V6(SADDR6(sa),					\
+		  &conf.c_match_mask6, 				\
+		  from, rcpt, bucket_count)			\
+   : 0)
+
+#else /* AF_INET6 */
+
+#define BUCKET_HASH(sa, from, rcpt, bucket_count) 		\
+  (sa->sa_family == AF_INET ?					\
+   BUCKET_HASH_V4(SADDR4(sa), 					\
+		  &conf.c_match_mask,				\
+		  from, rcpt, bucket_count)			\
+   : 0)
+
+#endif
+
+struct smtp_reply {
+	int sr_whitelist;
+	int sr_nowhitelist;
+	time_t sr_elapsed;
+	time_t sr_remaining;
+	int sr_acl_line;
+	time_t sr_delay;
+	time_t sr_autowhite;
+	char *sr_code;
+	char *sr_ecode;
+	char *sr_msg;
+	char *sr_msg_x;
+	char *sr_report;
+	char *sr_report_x;
+	sfsistat sr_retcode;
+	int sr_nmatch;
+	char **sr_pmatch;
+};
+
+struct rcpt {
+	char r_addr[ADDRLEN + 1];
+	LIST_ENTRY(rcpt) r_list;
+};
+
+struct header {
+	char *h_line;
+	TAILQ_ENTRY(header) h_list;
+};
+
+struct body {
+	char *b_lines;
+	TAILQ_ENTRY(body) b_list;
+};
+
 struct mlfi_priv {
+	SMFICTX *priv_ctx;
 	sockaddr_t priv_addr;
 	socklen_t priv_addrlen;
 	char priv_hostname[ADDRLEN + 1];
 	char priv_helo[ADDRLEN + 1];
 	char priv_from[ADDRLEN + 1];
-	char priv_rcpt[ADDRLEN + 1];
-	time_t priv_elapsed;
-	int priv_whitelist;
+	LIST_HEAD(, rcpt) priv_rcpt;
+	char *priv_cur_rcpt;
+	int priv_rcptcount;
+	TAILQ_HEAD(, header) priv_header;
+	TAILQ_HEAD(, body) priv_body;
+#ifdef USE_GEOIP
+	const char *priv_ccode;
+#endif
+	size_t priv_msgcount;
+	char *priv_buf;
+	size_t priv_buflen;
 	char *priv_queueid;
 	int priv_delayed_reject;
-	time_t priv_remaining;
+	struct smtp_reply priv_sr;
+	time_t priv_max_elapsed;
+	int priv_last_whitelist;
+#ifdef USE_CURL
+	LIST_HEAD(, urlcheck_prop) priv_prop;
+#endif
+#ifdef USE_DNSRBL
+	LIST_HEAD(, dnsrbl_list) priv_dnsrbl;
+#endif
 };
 
 sfsistat mlfi_connect(SMFICTX *, char *, _SOCK_ADDR *);
 sfsistat mlfi_helo(SMFICTX *, char *);
 sfsistat mlfi_envfrom(SMFICTX *, char **);
 sfsistat mlfi_envrcpt(SMFICTX *, char **);
+sfsistat mlfi_header(SMFICTX *, char *, char *);
+sfsistat mlfi_body(SMFICTX *, unsigned char *, size_t);
 sfsistat mlfi_eom(SMFICTX *);
 sfsistat mlfi_close(SMFICTX *);
 void usage(char *);
@@ -105,18 +217,22 @@ struct in_addr *prefix2mask4(int, struct in_addr *);
 #ifdef AF_INET6
 struct in6_addr *prefix2mask6(int, struct in6_addr *);
 #endif
-void cleanup_sock(char *);
 void unmappedaddr(struct sockaddr *, socklen_t *);
 void final_dump(void);
 int main(int, char **);
+void mg_log(int, char *, ...);
+char *strncpy_rmsp(char *, char *, size_t);
+char *fstring_expand(struct mlfi_priv *, 
+    char *, const char *);
+char *fstring_escape(char *);
 
-#if (defined(HAVE_SPF) || defined(HAVE_SPF_ALT))
-#define MLFI_HELO mlfi_helo
+#ifdef HAVE_STRLCAT
+/* #include <string.h> */
+#define mystrlcat strlcat
 #else
-#define MLFI_HELO NULL
+size_t mystrlcat(char *, const char *src, size_t size);
 #endif
 
-
 /*
  * Locking management
  */
@@ -140,6 +256,16 @@ int main(int, char **);
 	}								  \
 }
 
+#define TSS_SET(key, val) do {						  \
+	int err;							  \
+									  \
+	if ((err = pthread_setspecific(key, val)) != 0) {		  \
+		mg_log(LOG_ERR, "%s:%d pthread_setspecific failed: %s",	  \
+		    __FILE__, __LINE__, strerror(err));			  \
+		exit(EX_SOFTWARE);					  \
+	}								  \
+} while (/*CONSTCOND*/ 0)
+
 /*
  * There is a bug in GNU pth-2.0.0 that will cause a spurious EPERM
  * error when a thread releases a read lock that has been shared by
@@ -167,14 +293,6 @@ int main(int, char **);
 }
 #endif
 
-/*
- * Some systems don't know about LOG_PERROR. By defining it
- * to zero, we make it nilpotent
- */
-#ifdef HAVE_MISSING_LOG_PERROR
-#define LOG_PERROR 0
-#endif
-
 #ifdef HAVE_MISSING_TIMERADD
 #define	timeradd(tvp, uvp, vvp)						\
 	do {								\
@@ -199,9 +317,21 @@ int main(int, char **);
 #define ADD_REASON(whystr, reason)					\
 	{								\
 		if (whystr[0] != '\0')					\
-			strncat(whystr, ", ", sizeof(whystr));		\
-		strncat(whystr, reason, sizeof(whystr));		\
+			mystrlcat(whystr, ", ", sizeof(whystr));	\
+		mystrlcat(whystr, reason, sizeof(whystr));		\
 	}
 
+/*
+ * Due to race conditions in the libmilter shipped with sendmail <= 8.13.8,
+ * the whole process may die after receiving a signal.
+ * It makes impossible the final dump. Apply the following patch ASAP:
+ * http://www.j10n.org/files/libmilter-8.13.8-signal.patch
+ *
+ * If you don't want to apply it, the following knob enables an uncertain
+ * effort to workaround the bug. Do not ask me about this.
+ * 
+ */
+/* #define WORKAROUND_LIBMILTER_RACE_CONDITION */
+
 #endif /* _MILTER_GREYLIST_H_ */
 
